@@ -1,11 +1,16 @@
 #!/bin/bash
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.143 2003/10/13 07:43:38 carpaski Exp $
+# $Header: /home/cvsroot/gentoo-src/portage/bin/ebuild.sh,v 1.154 2004/02/04 22:40:21 carpaski Exp $
 
-if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
-	if [ -f ${T}/successful ]; then
-		rm -f ${T}/successful
+SANDBOX_PREDICT="${SANDBOX_PREDICT}:/proc/self/maps:/dev/console:/usr/lib/portage/pym"
+SANDBOX_WRITE="${SANDBOX_WRITE}:/dev/shm:${PORTAGE_TMPDIR}"
+SANDBOX_READ="${SANDBOX_READ}:/dev/shm:${PORTAGE_TMPDIR}"
+
+
+if [ "$*" != "depend" ] && [ "$*" != "clean" ] && [ "$*" != "nofetch" ]; then
+	if [ -f "${T}/successful" ]; then
+		rm -f "${T}/successful"
 	fi
 
 	# Hurray for per-ebuild logging.
@@ -13,19 +18,16 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 		if [ -z "${PORT_LOGGING}" ]; then
 			export PORT_LOGGING=1
 			export SANDBOX_WRITE="$SANDBOX_WRITE:${PORT_LOGDIR}"
-			install -d ${PORT_LOGDIR} &>/dev/null
-			chown root:portage ${PORT_LOGDIR} &>/dev/null
-			chmod g+rwxs ${PORT_LOGDIR} &> /dev/null
+			install -d "${PORT_LOGDIR}" &>/dev/null
+			chown root:portage "${PORT_LOGDIR}" &>/dev/null
+			chmod g+rwxs "${PORT_LOGDIR}" &> /dev/null
 			touch "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
 			chmod g+w "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log" &> /dev/null
 			echo "$*" >> "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
-			$0 $* 2>&1 | tee -a "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
-			if [ "$?" != "0" ]; then
-				rm -f ${T}/successful
-				exit 1
-			fi
-			if [ -f ${T}/successful ]; then
-				rm -f ${T}/successful
+			{ $0 $* || rm -f "${T}/successful" ; } 2>&1 \
+			  | tee -i -a "${PORT_LOGDIR}/${LOG_COUNTER}-${PF}.log"
+			if [ -f "${T}/successful" ]; then
+				rm -f "${T}/successful"
 				exit 0
 			else
 				exit 1
@@ -47,7 +49,7 @@ fi
 unalias -a
 
 # Unset some variables that break things.
-unset GZIP BZIP BZIP2 CDPATH
+unset GZIP BZIP BZIP2 CDPATH GREP_OPTIONS GREP_COLOR
 
 # We need this next line for "die" and "assert". It expands 
 # It _must_ preceed all the calls to die and assert.
@@ -84,22 +86,48 @@ esyslog() {
 	return 0
 }
 
+
 use() {
-	local x
-	for x in ${USE}; do
-		if [ "${x}" == "${1}" ]; then
-			if [ -r /dev/fd/1 ]; then
-				tty --quiet < /dev/stdout || echo "${x}"
-			else
-			  echo "${x}"
-			fi
-			return 0
-		fi
-	done
+	if useq ${1}; then
+		echo "${1}"
+		return 0
+	fi
 	return 1
 }
 
+useq() {
+	local u="${1}"
+	local neg=0
+	if [ "${u:0:1}" == "!" ]; then
+		u="${u:1}"
+		neg=1
+	fi
+	local x
+	for x in ${USE}; do
+		if [ "${x}" == "${u}" ]; then
+			if [ ${neg} -eq 1 ]; then
+				return 1
+			else
+				return 0
+			fi
+		fi
+	done
+	if [ ${neg} -eq 1 ]; then
+		return 0
+	else
+		return 1
+	fi
+}
+
 has() {
+	if hasq "$@"; then
+		echo "${1}"
+		return 0
+	fi
+	return 1
+}
+
+hasq() {
 	local x
 
 	local me=$1
@@ -109,17 +137,6 @@ has() {
 	# Logging kills all this anyway. Everything becomes a pipe. --NJ
 	for x in "$@"; do
 		if [ "${x}" == "${me}" ]; then
-			if [ -r /proc/self/fd/1 ]; then
-				tty --quiet < /proc/self/fd/1 || echo "${x}"
-			elif [ -r /dev/fd/1 ]; then
-				echo "/dev/fd/1" >&2
-				tty --quiet < /dev/fd/1 || echo "${x}"
-			elif [ -r /dev/stdout ]; then
-				echo "/dev/stdout" >&2
-				tty --quiet < /dev/stdout || echo "${x}"
-			else
-				echo "${x}"
-			fi
 			return 0
 		fi
 	done
@@ -127,6 +144,7 @@ has() {
 }
 
 has_version() {
+	[ "${EBUILD_PHASE}" == "depend" ] && echo "has_version() in global scope: ${CATEGORY}/$PF" >&2
 	# return shell-true/shell-false if exists.
 	# Takes single depend-type atoms.
 	if /usr/lib/portage/bin/portageq 'has_version' "${ROOT}" "$1"; then
@@ -136,7 +154,13 @@ has_version() {
 	fi
 }
 
+portageq() {
+	[ "${EBUILD_PHASE}" == "depend" ] && echo "portageq in global scope: ${CATEGORY}/$PF" >&2
+	/usr/lib/portage/bin/portageq "$@"
+}
+
 best_version() {
+	[ "${EBUILD_PHASE}" == "depend" ] && echo "best_version() in global scope: ${CATEGORY}/$PF" >&2
 	# returns the best/most-current match.
 	# Takes single depend-type atoms.
 	/usr/lib/portage/bin/portageq 'best_version' "${ROOT}" "$1"
@@ -159,7 +183,7 @@ use_with() {
 		UWORD="$1"
 	fi
 	
-	if use $1 &>/dev/null; then
+	if useq $1; then
 		echo "--with-${UWORD}${UW_SUFFIX}"
 		return 0
 	else
@@ -185,7 +209,7 @@ use_enable() {
 		UWORD="$1"
 	fi
 	
-	if use $1 &>/dev/null; then
+	if useq $1; then
 		echo "--enable-${UWORD}${UE_SUFFIX}"
 		return 0
 	else
@@ -194,8 +218,6 @@ use_enable() {
 	fi
 }
 
-#The following diefunc() and aliases come from Aron Griffis -- an excellent bash coder -- thanks! 
-
 diefunc() {
 	local funcname="$1" lineno="$2" exitcode="$3"
 	shift 3
@@ -240,9 +262,14 @@ keepdir()
 {
 	dodir "$@"
 	local x
-	for x in "$@"; do
-		touch ${D}/${x}/.keep || die "Failed to create .keep in ${D}/${x}"
-	done
+	if [ "$1" == "-R" ] || [ "$1" == "-r" ]; then
+		shift
+		find "$@" -type d -printf "${D}/%p/.keep\n" | tr "\n" "\0" | $XARGS -0 -n100 touch || die "Failed to recursive create .keep files"
+	else
+		for x in "$@"; do
+			touch "${D}/${x}/.keep" || die "Failed to create .keep in ${D}/${x}"
+		done
+	fi
 }
 
 # the sandbox is disabled by default except when overridden in the relevant stages
@@ -289,29 +316,29 @@ unpack() {
 
 		case "${x##*.}" in
 			tar) 
-				tar ${tarvars} -xf ${DISTDIR}/${x} || die "$myfail"
+				tar ${tarvars} -xf "${DISTDIR}/${x}" || die "$myfail"
 				;;
 			tgz) 
-				tar ${tarvars} -xzf ${DISTDIR}/${x} || die "$myfail"
+				tar ${tarvars} -xzf "${DISTDIR}/${x}" || die "$myfail"
 				;;
 			tbz2) 
-				bzip2 -dc ${DISTDIR}/${x} | tar ${tarvars} -xf - || die "$myfail"
+				bzip2 -dc "${DISTDIR}/${x}" | tar ${tarvars} -xf - || die "$myfail"
 				;;
 			ZIP|zip) 
-				unzip -qo ${DISTDIR}/${x} || die "$myfail"
+				unzip -qo "${DISTDIR}/${x}" || die "$myfail"
 				;;
 			gz|Z|z) 
 				if [ "${y}" == "tar" ]; then
-					tar ${tarvars} -xzf ${DISTDIR}/${x} || die "$myfail"
+					tar ${tarvars} -xzf "${DISTDIR}/${x}" || die "$myfail"
 				else
-					gzip -dc ${DISTDIR}/${x} > ${x%.*} || die "$myfail"
+					gzip -dc "${DISTDIR}/${x}" > ${x%.*} || die "$myfail"
 				fi
 				;;
 			bz2) 
 				if [ "${y}" == "tar" ]; then
-					bzip2 -dc ${DISTDIR}/${x} | tar ${tarvars} -xf - || die "$myfail"
+					bzip2 -dc "${DISTDIR}/${x}" | tar ${tarvars} -xf - || die "$myfail"
 				else
-					bzip2 -dc ${DISTDIR}/${x} > ${x%.*} || die "$myfail"
+					bzip2 -dc "${DISTDIR}/${x}" > ${x%.*} || die "$myfail"
 				fi
 				;;
 			*)
@@ -417,19 +444,6 @@ pkg_postrm()
 	return
 }
 
-try() {
-	env "$@"
-	if [ "$?" != "0" ]
-	then
-		echo 
-		echo '!!! '"ERROR: the $1 command did not complete successfully."
-		echo '!!! '"(\"$*\")"
-		echo '!!! '"Since this is a critical task, ebuild will be stopped."
-		echo
-		exit 1
-	fi
-}
-
 # Used to generate the /lib/cpp and /usr/bin/cc wrappers
 gen_wrapper() {
 	cat > $1 << END
@@ -462,7 +476,7 @@ dyn_setup()
 			fi
 		fi
 	fi
-	pkg_setup || die "pkg_setup function failed; exiting."
+	pkg_setup
 }
 
 dyn_unpack() {
@@ -480,7 +494,7 @@ dyn_unpack() {
 				break
 			fi
 		done
-		if [ ${EBUILD} -nt "${WORKDIR}" ]; then
+		if [ "${EBUILD}" -nt "${WORKDIR}" ]; then
 			echo ">>> ${EBUILD} has been updated; recreating WORKDIR..."
 			newstuff="yes"
 			rm -rf "${WORKDIR}"
@@ -501,30 +515,30 @@ dyn_unpack() {
 	[ -d "$WORKDIR" ] && cd "${WORKDIR}"
 	echo ">>> Unpacking source..."
 	src_unpack
-	touch ${BUILDDIR}/.unpacked || die "IO Failure -- Failed 'touch .unpacked' in BUILDIR"
+	touch "${BUILDDIR}/.unpacked" || die "IO Failure -- Failed 'touch .unpacked' in BUILDIR"
 	echo ">>> Source unpacked."
-	cd $BUILDDIR
+	cd "$BUILDDIR"
 	trap SIGINT SIGQUIT
 }
 
 dyn_clean() {
-	rm -rf ${BUILDDIR}/image
-	rm -rf ${BUILDDIR}/build-info
+	rm -rf "${BUILDDIR}/image"
+	rm -rf "${BUILDDIR}/build-info"
 
 	if ! has keeptemp $FEATURES; then
-		rm -rf ${T}/*
+		rm -rf "${T}"/*
 	else
-		mv ${T}/environment ${T}/environment.keeptemp
+		mv "${T}/environment" "${T}/environment.keeptemp"
 	fi
 
 	if ! has keepwork $FEATURES; then
-		rm -rf ${BUILDDIR}/.compiled
-		rm -rf ${BUILDDIR}/.unpacked
+		rm -rf "${BUILDDIR}/.compiled"
+		rm -rf "${BUILDDIR}/.unpacked"
 		rm -rf "${WORKDIR}"
 	fi
 
-	if [ -f ${BUILDDIR}/.unpacked ]; then
-		find ${BUILDDIR} -type d ! -regex "^${WORKDIR}" | sort -r | tr "\n" "\0" | $XARGS -0 rmdir &>/dev/null
+	if [ -f "${BUILDDIR}/.unpacked" ]; then
+		find "${BUILDDIR}" -type d ! -regex "^${WORKDIR}" | sort -r | tr "\n" "\0" | $XARGS -0 rmdir &>/dev/null
 	fi
 	true
 }
@@ -637,27 +651,27 @@ abort_handler() {
 
 abort_compile() {
 	abort_handler "src_compile" $1
-	rm -f ${BUILDDIR}/.compiled
+	rm -f "${BUILDDIR}/.compiled"
 	exit 1
 }
 
 abort_unpack() {
 	abort_handler "src_unpack" $1
-	rm -f ${BUILDDIR}/.unpacked
-	rm -rf ${BUILDDIR}/work
+	rm -f "${BUILDDIR}/.unpacked"
+	rm -rf "${BUILDDIR}/work"
 	exit 1
 }
 
 abort_package() {
 	abort_handler "dyn_package" $1
-	rm -f ${BUILDDIR}/.packaged
-	rm -f ${PKGDIR}/All/${PF}.t*
+	rm -f "${BUILDDIR}/.packaged"
+	rm -f "${PKGDIR}"/All/${PF}.t*
 	exit 1
 }
 
 abort_install() {
 	abort_handler "src_install" $1
-	rm -rf ${BUILDDIR}/image
+	rm -rf "${BUILDDIR}/image"
 	exit 1
 }
 
@@ -670,7 +684,7 @@ dyn_compile() {
 	[ "${LDFLAGS-unset}"     != "unset" ] && export LDFLAGS
 	[ "${ASFLAGS-unset}"     != "unset" ] && export ASFLAGS
 
-	[ "${DISTCC_DIR-unset}"  == "unset" ] && export DISTCC_DIR="${PORT_TMPDIR}/.distcc"
+	[ "${DISTCC_DIR-unset}"  == "unset" ] && export DISTCC_DIR="${PORTAGE_TMPDIR}/.distcc"
 	[ ! -z "${DISTCC_DIR}" ] && addwrite "${DISTCC_DIR}"
 
 	if has noauto $FEATURES &>/dev/null && [ ! -f ${BUILDDIR}/.unpacked ]; then
@@ -692,11 +706,11 @@ dyn_compile() {
 		sleep 3
 	fi
 
-	cd ${BUILDDIR}
+	cd "${BUILDDIR}"
 	if [ ! -e "build-info" ];	then
 		mkdir build-info
 	fi
-	cp ${EBUILD} build-info/${PF}.ebuild
+	cp "${EBUILD}" "build-info/${PF}.ebuild"
 	
 	if [ ${BUILDDIR}/.compiled -nt "${WORKDIR}" ]; then
 		echo ">>> It appears that ${PN} is already compiled; skipping."
@@ -715,7 +729,7 @@ dyn_compile() {
 	export PWORKDIR="$WORKDIR"
 	src_compile 
 	#|| abort_compile "fail" 
-	cd ${BUILDDIR}
+	cd "${BUILDDIR}"
 	touch .compiled
 	cd build-info
 	echo "$CBUILD"   > CBUILD
@@ -727,6 +741,7 @@ dyn_compile() {
 	echo "$CXXFLAGS" > CXXFLAGS
 	echo "$DEPEND"   > DEPEND
 	echo "$IUSE"     > IUSE
+	echo "$PKGUSE"   > PKGUSE
 	echo "$LICENSE"  > LICENSE
 	echo "$CATEGORY" > CATEGORY
 	echo "$PDEPEND"  > PDEPEND
@@ -736,7 +751,7 @@ dyn_compile() {
 	echo "$SLOT"     > SLOT
 	echo "$USE"      > USE
 	set | bzip2 -9 - > environment.bz2
-	cp ${EBUILD} ${PF}.ebuild
+	cp "${EBUILD}" "${PF}.ebuild"
 	if has nostrip $FEATURES $RESTRICT; then
 		touch DEBUGBUILD
 	fi
@@ -745,27 +760,27 @@ dyn_compile() {
 
 dyn_package() {
 	trap "abort_package" SIGINT SIGQUIT
-	cd ${BUILDDIR}/image
+	cd "${BUILDDIR}/image"
 	tar cpvf - ./ | bzip2 -f > ../bin.tar.bz2 || die "Failed to create tarball"
 	cd ..
 	xpak build-info inf.xpak
-	tbz2tool join bin.tar.bz2 inf.xpak ${PF}.tbz2
-	mv ${PF}.tbz2 ${PKGDIR}/All || die "Failed to move tbz2 to ${PKGDIR}/All"
+	tbz2tool join bin.tar.bz2 inf.xpak "${PF}.tbz2"
+	mv "${PF}.tbz2" "${PKGDIR}/All" || die "Failed to move tbz2 to ${PKGDIR}/All"
 	rm -f inf.xpak bin.tar.bz2
-	if [ ! -d ${PKGDIR}/${CATEGORY} ]; then
-		install -d ${PKGDIR}/${CATEGORY}
+	if [ ! -d "${PKGDIR}/${CATEGORY}" ]; then
+		install -d "${PKGDIR}/${CATEGORY}"
 	fi
-	ln -sf ../All/${PF}.tbz2 ${PKGDIR}/${CATEGORY}/${PF}.tbz2 || die "Failed to create symlink in ${PKGDIR}/${CATEGORY}"
+	ln -sf "../All/${PF}.tbz2" "${PKGDIR}/${CATEGORY}/${PF}.tbz2" || die "Failed to create symlink in ${PKGDIR}/${CATEGORY}"
 	echo ">>> Done."
-	cd ${BUILDDIR}
+	cd "${BUILDDIR}"
 	touch .packaged || die "Failed to 'touch .packaged' in ${BUILDDIR}"
 	trap SIGINT SIGQUIT
 }
 
 dyn_install() {
 	trap "abort_install" SIGINT SIGQUIT
-	rm -rf ${BUILDDIR}/image
-	mkdir ${BUILDDIR}/image
+	rm -rf "${BUILDDIR}/image"
+	mkdir "${BUILDDIR}/image"
 	if [ -d "${S}" ]; then
 		cd "${S}"
 	fi
@@ -780,14 +795,14 @@ dyn_install() {
 	src_install 
 	#|| abort_install "fail"
 	prepall
-	cd ${D}
+	cd "${D}"
 
 	declare -i UNSAFE=0
-	for i in $(find ${D}/ -type f -perm -2002); do
+	for i in $(find "${D}/" -type f -perm -2002); do
 		UNSAFE=$(($UNSAFE + 1))
 		echo "UNSAFE SetGID: $i"
 	done
-	for i in $(find ${D}/ -type f -perm -4002); do
+	for i in $(find "${D}/" -type f -perm -4002); do
 		UNSAFE=$(($UNSAFE + 1))
 		echo "UNSAFE SetUID: $i"
 	done
@@ -796,11 +811,11 @@ dyn_install() {
 		die "There are ${UNSAFE} unsafe files. Portage will not install them."
 	fi
 	
-	find ${D}/ -user  portage -print0 | $XARGS -0 -n100 chown root
+	find "${D}/" -user  portage -print0 | $XARGS -0 -n100 chown root
 	if [ "$USERLAND" == "BSD" ]; then
-		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp wheel
+		find "${D}/" -group portage -print0 | $XARGS -0 -n100 chgrp wheel
 	else
-		find ${D}/ -group portage -print0 | $XARGS -0 -n100 chgrp root
+		find "${D}/" -group portage -print0 | $XARGS -0 -n100 chgrp root
 	fi
 
 	echo ">>> Completed installing into ${D}"
@@ -810,41 +825,73 @@ dyn_install() {
 }
 
 dyn_preinst() {
-	pkg_preinst
-
 	# set IMAGE depending if this is a binary or compile merge
-	[ "${EMERGE_FROM}" == "binary" ] && IMAGE=${PKG_TMPDIR}/${PF} \
+	[ "${EMERGE_FROM}" == "binary" ] && IMAGE=${PKG_TMPDIR}/${PF}/bin \
 					|| IMAGE=${D}
 
+	pkg_preinst
+
 	# remove man pages
 	if has noman $FEATURES; then
-		rm -fR ${IMAGE}/usr/share/man
+		rm -fR "${IMAGE}/usr/share/man"
 	fi
 
 	# remove info pages
 	if has noinfo $FEATURES; then
-		rm -fR ${IMAGE}/usr/share/info
+		rm -fR "${IMAGE}/usr/share/info"
 	fi
 
 	# remove docs
 	if has nodoc $FEATURES; then
-		rm -fR ${IMAGE}/usr/share/doc
+		rm -fR "${IMAGE}/usr/share/doc"
 	fi
 
 	# Smart FileSystem Permissions
 	if has sfperms $FEATURES; then
 		for i in $(find ${IMAGE}/ -type f -perm -4000); do
 			ebegin ">>> SetUID: [chmod go-r] $i "
-			chmod go-r $i
+			chmod go-r "$i"
 			eend $?
 		done
 		for i in $(find ${IMAGE}/ -type f -perm -2000); do
 			ebegin ">>> SetGID: [chmod o-r] $i "
-			chmod o-r $i
+			chmod o-r "$i"
 			eend $?
 		done
 	fi
 
+	# total suid control.
+	if has suidctl $FEATURES > /dev/null ; then
+		sfconf=/etc/portage/suidctl.conf
+		echo ">>> Preforming suid scan in ${IMAGE}"
+		for i in $(find ${IMAGE}/ -type f \( -perm -4000 -o -perm -2000 \) ); do
+			if [ -s "${sfconf}" ]; then
+				suid="`grep ^${i/${IMAGE}/}$ ${sfconf}`"
+				if [ "${suid}" = "${i/${IMAGE}/}" ]; then
+					echo "- ${i/${IMAGE}/} is an approved suid file"
+				else
+					for x in 5 4 3 2 1 0; do echo -ne "\a"; sleep 0.25 ; done
+					echo ">>> Removing sbit on non registered ${i/${IMAGE}/}"
+					echo -ne "\a"
+					chmod ugo-s "${i}"
+					grep ^#${i/${IMAGE}/}$ ${sfconf} > /dev/null || {
+						# sandbox prevents us from writing directly
+						# to files outside of the sandbox, but this
+						# can easly be bypassed using the addwrite() function
+						addwrite "${sfconf}"
+						echo ">>> Appending commented out entry to ${sfconf} for ${PF}"
+						ls -ldh "${i}" | awk '{print "## "$0}' | sed s:"${IMAGE}"::g >> ${sfconf}
+						echo "#${i/${IMAGE}/}" >> ${sfconf}
+						# no delwrite() eh?
+						# delwrite ${sconf}
+					}
+				fi
+			else
+				echo "suidctl feature set but you are lacking a ${sfconf}"
+			fi
+		done
+	fi
+
 	# SELinux file labeling (needs to always be last in dyn_preinst)
 	if use selinux; then
 		# only attempt to label if setfiles is executable
@@ -852,13 +899,13 @@ dyn_preinst() {
 		if [ -f /selinux/context -a -x /usr/sbin/setfiles ]; then
 			echo ">>> Setting SELinux security labels"
 			if [ -f ${POLICYDIR}/file_contexts/file_contexts ]; then
-				cp -f ${POLICYDIR}/file_contexts/file_contexts ${T}
+				cp -f "${POLICYDIR}/file_contexts/file_contexts" "${T}"
 			else
-				make -C ${POLICYDIR} FC=${T}/file_contexts ${T}/file_contexts
+				make -C "${POLICYDIR}" FC=${T}/file_contexts "${T}/file_contexts"
 			fi
 
 			addwrite /selinux/context
-			/usr/sbin/setfiles -r ${IMAGE} ${T}/file_contexts ${IMAGE} \
+			/usr/sbin/setfiles -r "${IMAGE}" "${T}/file_contexts" "${IMAGE}" \
 				|| die "Failed to set SELinux security labels."
 		else
 			# nonfatal, since merging can happen outside a SE kernel
@@ -870,7 +917,7 @@ dyn_preinst() {
 }
 
 dyn_spec() {
-	tar czf /usr/src/redhat/SOURCES/${PF}.tar.gz ${O}/${PF}.ebuild ${O}/files || die "Failed to create base rpm tarball."
+	tar czf "/usr/src/redhat/SOURCES/${PF}.tar.gz" "${O}/${PF}.ebuild" "${O}/files" || die "Failed to create base rpm tarball."
 
 	cat <<__END1__ > ${PF}.spec
 Summary: ${DESCRIPTION}
@@ -903,8 +950,8 @@ __END1__
 
 dyn_rpm() {
 	dyn_spec
-	rpm -bb ${PF}.spec || die "Failed to integrate rpm spec file"
-	install -D /usr/src/redhat/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm ${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm || die "Failed to move rpm"
+	rpm -bb "${PF}.spec" || die "Failed to integrate rpm spec file"
+	install -D "/usr/src/redhat/RPMS/i386/${PN}-${PV}-${PR}.i386.rpm" "${RPMDIR}/${CATEGORY}/${PN}-${PV}-${PR}.rpm" || die "Failed to move rpm"
 }
 
 dyn_help() {
@@ -980,9 +1027,9 @@ debug-print() {
 		fi
 		
 		# default target
-		echo "$1" >> ${T}/eclass-debug.log
+		echo "$1" >> "${T}/eclass-debug.log"
 		# let the portage user own/write to this file
-		chmod g+w ${T}/eclass-debug.log &>/dev/null
+		chmod g+w "${T}/eclass-debug.log" &>/dev/null
 		
 		shift
 	done
@@ -1009,6 +1056,13 @@ inherit() {
 	fi
 
 	local location
+	local PECLASS
+
+	local B_IUSE
+	local B_DEPEND
+	local B_RDEPEND
+	local B_CDEPEND
+	local B_PDEPEND
 
 	while [ "$1" ]; do
 		location="${ECLASSDIR}/${1}.eclass"
@@ -1075,10 +1129,9 @@ inherit() {
 		#turn on glob expansion
  		set +f
 		
-		has $1 $INHERITED || export INHERITED="$INHERITED $1"
+		hasq $1 $INHERITED || export INHERITED="$INHERITED $1"
 
 		export ECLASS="$PECLASS"
-		unset PECLASS
 
 		shift
 	done
@@ -1096,7 +1149,7 @@ EXPORT_FUNCTIONS() {
 	fi
 	while [ "$1" ]; do
 		debug-print "EXPORT_FUNCTIONS: ${1} -> ${ECLASS}_${1}" 
-		eval "$1() { ${ECLASS}_$1 ; }" > /dev/null
+		eval "$1() { ${ECLASS}_$1 "\$@" ; }" > /dev/null
 		shift
 	done
 }
@@ -1212,12 +1265,18 @@ if [ "$*" != "depend" ] && [ "$*" != "clean" ]; then
 
 		[ -z "${CCACHE_DIR}" ] && export CCACHE_DIR="/root/.ccache"
 
-		addread ${CCACHE_DIR}
-		addwrite ${CCACHE_DIR}
+		addread "${CCACHE_DIR}"
+		addwrite "${CCACHE_DIR}"
 
 		[ -z "${CCACHE_SIZE}" ] && export CCACHE_SIZE="2G"
 		ccache -M ${CCACHE_SIZE} &> /dev/null
 	fi
+
+	# XXX: Load up the helper functions.
+#	for X in /usr/lib/portage/bin/functions/*.sh; do
+#		source ${X} || die "Failed to source ${X}"
+#	done
+	
 else
 
 killparent() {
@@ -1234,6 +1293,7 @@ export S=${WORKDIR}/${P}
 unset   DEPEND   RDEPEND   CDEPEND   PDEPEND
 unset E_DEPEND E_RDEPEND E_CDEPEND E_PDEPEND
 
+export EBUILD_PHASE="$*"
 source ${EBUILD} || die "error sourcing ebuild"
 [ -z "${ERRORMSG}" ] || die "${ERRORMSG}"
 
@@ -1272,6 +1332,7 @@ for myarg in $*; do
 	case $myarg in
 	nofetch)
 		pkg_nofetch
+		exit 1
 		;;
 	prerm|postrm|postinst|config)
 		export SANDBOX_ON="0"
@@ -1328,13 +1389,18 @@ for myarg in $*; do
 	depend)
 		export SANDBOX_ON="0"
 		set -f
-		#the extra `echo` commands remove newlines
-		dbkey=${PORTAGE_CACHEDIR}/${CATEGORY}/${PF}
-		if [ ! -d ${PORTAGE_CACHEDIR}/${CATEGORY} ]; then
-			install -d -g ${PORTAGE_GID} -m4775 ${PORTAGE_CACHEDIR}/${CATEGORY}
+
+		# Handled in portage.py now
+		#dbkey=${PORTAGE_CACHEDIR}/${CATEGORY}/${PF}
+
+		if [ ! -d "${dbkey%/*}" ]; then
+			install -d -g ${PORTAGE_GID} -m2775 "${dbkey%/*}"
 		fi
+
 		# Make it group writable. 666&~002==664
 		umask 002
+
+		#the extra `echo` commands remove newlines
 		echo `echo "$DEPEND"`       > $dbkey
 		echo `echo "$RDEPEND"`     >> $dbkey
 		echo `echo "$SLOT"`        >> $dbkey
@@ -1360,20 +1426,21 @@ for myarg in $*; do
 		exit 1
 		;;
 	esac
-	if [ $? -ne 0 ]; then
-		exit 1
-	fi
+
+	#if [ $? -ne 0 ]; then
+	#	exit 1
+	#fi
 done
 
 if [ "$myarg" != "clean" ]; then
 	# Save current environment and touch a success file. (echo for success)
 	umask 002
-	set | egrep -v "^SANDBOX_" > ${T}/environment 2>/dev/null
-	chown portage:portage ${T}/environment &>/dev/null
-	chmod g+w ${T}/environment &>/dev/null
+	set | egrep -v "^SANDBOX_" > "${T}/environment" 2>/dev/null
+	chown portage:portage "${T}/environment" &>/dev/null
+	chmod g+w "${T}/environment" &>/dev/null
 fi
-touch ${T}/successful  &>/dev/null
-chown portage:portage ${T}/successful &>/dev/null
-chmod g+w ${T}/successful &>/dev/null
+touch "${T}/successful"  &>/dev/null
+chown portage:portage "${T}/successful" &>/dev/null
+chmod g+w "${T}/successful" &>/dev/null
 
 exit 0
