@@ -1,7 +1,7 @@
 #!/usr/bin/env python2.2
 # Copyright 1999-2003 Gentoo Technologies, Inc.
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /home/cvsroot/gentoo-src/portage/bin/emergehelp.py,v 1.16 2003/08/16 07:28:22 carpaski Exp $
+# $Header: /home/cvsroot/gentoo-src/portage/bin/emergehelp.py,v 1.20 2004/02/04 22:40:21 carpaski Exp $
 
 import os,sys
 from output import *
@@ -13,9 +13,9 @@ def shorthelp():
 	print "   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] [ "+turquoise("ebuildfile")+" | "+turquoise("tbz2file")+" | "+turquoise("dependency")+" ] [ ... ]"
 	print "   "+turquoise("emerge")+" [ "+green("options")+" ] [ "+green("action")+" ] < "+turquoise("system")+" | "+turquoise("world")+" >"
 	print "   "+turquoise("emerge")+" < "+turquoise("sync")+" | "+turquoise("info")+" >"
-	print "   "+turquoise("emerge")+" "+turquoise("--resume")+" ["+green("--pretend")+" | "+green("--skipfirst")+"]"
+	print "   "+turquoise("emerge")+" "+turquoise("--resume")+" [ "+green("--pretend")+" | "+green("--ask")+" | "+green("--skipfirst")+" ]"
 	print "   "+turquoise("emerge")+" "+turquoise("help")+" [ "+green("system")+" | "+green("config")+" | "+green("sync")+" ] "
-	print bold("Options:")+" "+green("-")+"["+green("bcCdDefhikKlnoOpPsSuUvV")+"] ["+green("--oneshot")+"] ["+green("--noconfmem")+"]"
+	print bold("Options:")+" "+green("-")+"["+green("abcCdDefhikKlnoOpPsSuUvV")+"] ["+green("--oneshot")+"] ["+green("--noconfmem")+"]"
 	print      "                                    ["+green("--columns")+"] ["+green("--nospinner")+"]"
 	print bold("Actions:")+" [ "+green("clean")+" | "+green("depclean")+" | "+green("inject")+" | "+green("prune")+" | "+green("regen")+" | "+green("search")+" | "+green("unmerge")+" ]"
 	print
@@ -42,8 +42,7 @@ def help(myaction,myopts,havecolor=1):
 		print "              remove functionalities or prevent your system from working."
 		print "              The arguments can be in several different formats :"
 		print "              * world "
-		print "              * system "
-		print "              * /var/db/pkg/category/package-version, or"
+		print "              * system or"
 		print "              * 'dependency specification' (in single quotes is best.)"
 		print "              Here are a few examples of the dependency specification format:"
 		print "              "+bold("binutils")+" matches"
@@ -122,6 +121,14 @@ def help(myaction,myopts,havecolor=1):
 		print "              format described in the "+bold("clean")+" action above."
 		print
 		print turquoise("Options:")
+		print "       "+green("--ask")+" ("+green("-a")+" short option)"
+		print "              before performing the merge, display what ebuilds and tbz2s will"
+		print "              be installed, in the same format as when using --pretend; then"
+		print "              ask whether to continue with the merge or abort. Using --ask is"
+		print "              more efficient than using --pretend and then executing the same"
+		print "              command without --pretend, as dependencies will only need to be"
+		print "              calculated once."
+		print
 		print "       "+green("--buildpkg")+" ("+green("-b")+" short option)"
 		print "              tell emerge to build binary packages for all ebuilds processed"
 		print "              (in addition to actually merging the packages.  Useful for"
@@ -129,7 +136,7 @@ def help(myaction,myopts,havecolor=1):
 		print "              systems (build once, emerge tbz2s everywhere)."
 		print
 		print "       "+green("--buildpkgonly")+" ("+green("-B")+" short option)"
-		print "              Creates binary a binary package, but does not merge it to the"
+		print "              Creates a binary package, but does not merge it to the"
 		print "              system. This has the restriction that unsatisfied dependencies"
 		print "              must not exist for the desired package as they cannot be used if"
 		print "              they do not exist on the system."
@@ -203,9 +210,9 @@ def help(myaction,myopts,havecolor=1):
 		print "              Disables the spinner regardless of terminal type."
 		print
 		print "       "+green("--oneshot")
-		print "              Emerge as normal, but don't add packages to the world profile for"
-		print "              later updating. This prevents consideration of this package"
-		print "              unless this package is depended upon by another package."
+		print "              Emerge as normal, but don't add packages to the world profile."
+		print "              This package is will only be updated if it is depend on by"
+		print "              another package."
 		print
 		print "       "+green("--onlydeps")+" ("+green("-o")+" short option)"
 		print "              Only merge (or pretend to merge) the dependencies of the"
