@@ -1,8 +1,12 @@
 #!/usr/bin/python
 
 import os,sys,re
+sys.path = ["/usr/lib/portage/pym"]+sys.path
+
 from stat import *
 from output import *
+from portage import lockfile,unlockfile,VDB_PATH,root
+
 
 mylog = open("/var/log/emerge_fix-db.log", "a")
 def writemsg(msg):
@@ -31,7 +35,7 @@ except:
 	writemsg("ERROR: Real counter is invalid.\n")
 	real_counter = 0
 
-vardbdir = "/var/db/pkg/"
+vardbdir = root+VDB_PATH+"/"
 for cat in os.listdir(vardbdir):
 	catdir = vardbdir+cat+"/"
 	if not os.path.isdir(catdir):
@@ -126,19 +130,20 @@ for catpkg in bad.keys():
 
 	if bad[catpkg] == ["CONTENTS is missing", "SLOT is missing"]:
 		writemsg("%s: (possibly injected)\n%s\n" % (green(catpkg), mystr))
-		actions[catpkg] = "ignore"
+		actions[catpkg] = ["ignore"]
 	elif bad[catpkg] == ["SLOT is empty"]:
-		writemsg("%s: (old package) [emerge/delete]\n%s\n" % (yellow(catpkg), mystr))
-		actions[catpkg] = "remerge"
+		writemsg("%s: (old package) []\n%s\n" % (yellow(catpkg), mystr))
+		actions[catpkg] = ["remerge"]
 	else:
-		writemsg("%s: (damaged/invalid) [remerge exact]\n%s\n" % (red(catpkg), mystr))
-		actions[catpkg] = "remerge"
+		writemsg("%s: (damaged/invalid) []\n%s\n" % (red(catpkg), mystr))
+		actions[catpkg] = ["merge exact"]
 
 if (len(sys.argv) > 1) and (sys.argv[1] == "--fix"):
 	writemsg("These are only directions, at the moment.")
 	for catpkg in actions.keys():
 		action = actions[catpkg]
 		writemsg("We will now '%s' '%s'..." % (action, catpkg))
+		#if action == 
 else:
 	#writemsg("Run with '--fix' to attempt automatic correction.")
 	pass
