@@ -1,5 +1,5 @@
 #!/usr/bin/perl -w
-# $Header: /home/cvsroot/gentoo-src/portage/bin/g-cpan.pl,v 1.10 2003/05/24 05:40:37 jrray Exp $
+# $Header: /home/cvsroot/gentoo-src/portage/bin/g-cpan.pl,v 1.12 2004/01/22 05:51:45 carpaski Exp $
 
 # History: 
 
@@ -72,7 +72,6 @@ unless (@ARGV) {
     print "Feed me perl modules\n";
     exit;
 }
-
 # Set our temporary overlay directory for the scope of this run. By setting an overlay directory,
 # we bypass the predefined portage directory and allow portage to build a package outside of its
 # normal tree.
@@ -84,6 +83,8 @@ my $TMP_DEV_PERL_DIR = '/var/tmp/db/dev-perl';
 my $MAKECONF         = '/etc/make.conf';
 my ( $OVERLAY_DIR, $PORTAGE_DIR, $PORTAGE_DEV_PERL, $PORTAGE_DISTDIR ) = get_globals();
 
+my $arches = join( ' ', map { chomp; $_ } `cat $PORTAGE_DIR/profiles/arch.list` );
+
 #this should never find the dir, but just to be safe
 unless ( -d $tmp_overlay_dir ) {
     mkpath( [$tmp_overlay_dir], 1, 0755 )
@@ -189,7 +190,7 @@ HOMEPAGE="http://www.cpan.org/modules/by-authors/id/$modpath/\${P}.readme"
 
 SLOT="0"
 LICENSE="Artistic | GPL-2"
-KEYWORDS="alpha arm hppa mips ppc sparc x86"
+KEYWORDS="$arches"
 
 HERE
 
