@@ -1,11 +1,10 @@
 # getbinpkg.py -- Portage binary-package helper functions
 # Copyright 2003 Gentoo Technologies, Inc.
 # Distributed under the GNU Public License v2
-# Author: Nicholas Jones <carpaski@gentoo.org>
-# $Id: $
+# $Id: getbinpkg.py,v 1.5 2004/02/04 22:40:21 carpaski Exp $
 
 from output import *
-import htmllib,HTMLParser,string,formatter,sys,os,xpak,time,tempfile,cPickle
+import htmllib,HTMLParser,string,formatter,sys,os,xpak,time,tempfile,cPickle,base64
 
 try:
 	import ftplib
@@ -95,6 +94,18 @@ def create_conn(baseurl,conn=None):
 		password = None
 	del userpass
 
+	http_headers = {}
+	http_params = {}
+	if username and password:
+		http_headers = {
+			"Authorization": "Basic %s" %
+			  string.replace(
+			    base64.encodestring("%s:%s" % (username, password)),
+			    "\012",
+			    ""
+			  ),
+		}
+
 	if not conn:
 		if protocol == "https":
 			conn = httplib.HTTPSConnection(host)
@@ -112,7 +123,7 @@ def create_conn(baseurl,conn=None):
 		else:
 			raise NotImplementedError, "%s is not a supported protocol." % protocol
 
-	return (conn,protocol,address)
+	return (conn,protocol,address, http_params, http_headers)
 
 def make_ftp_request(conn, address, rest=None, dest=None):
 	"""(conn,address,rest) --- uses the conn object to request the data
@@ -249,18 +260,19 @@ def dir_get_list(baseurl,conn=None):
 	else:
 		keepconnection = 1
 
-	conn,protocol,address = create_conn(baseurl, conn)
+	conn,protocol,address,params,headers = create_conn(baseurl, conn)
 
+	listing = None
 	if protocol in ["http","https"]:
-		headers = {}
-		params  = {}
-		page,rc,msg = make_http_request(conn, address, params, headers)
-
+		page,rc,msg = make_http_request(conn,address,params,headers)
+		
 		if page:
 			parser = ParseLinks()
 			parser.feed(page)
 			del page
 			listing = parser.get_anchors()
+		else:
+			raise Exception, "Unable to get listing: %s" % (rc,msg)
 	elif protocol in ["ftp"]:
 		if address[-1] == '/':
 			olddir = conn.pwd()
@@ -288,11 +300,10 @@ def file_get_metadata(baseurl,conn=None, chunk_size=3000):
 	else:
 		keepconnection = 1
 
-	conn,protocol,address = create_conn(baseurl, conn)
+	conn,protocol,address,params,headers = create_conn(baseurl, conn)
 
 	if protocol in ["http","https"]:
-		headers = {"Range": "bytes=-"+str(chunk_size)}
-		params  = {}
+		headers["Range"] = "bytes=-"+str(chunk_size)
 		data,rc,msg = make_http_request(conn, address, params, headers)
 	elif protocol in ["ftp"]:
 		data,rc,msg = make_ftp_request(conn, address, -chunk_size)
@@ -361,11 +372,11 @@ def file_get_lib(baseurl,dest,conn=None):
 	else:
 		keepconnection = 1
 
-	conn,protocol,address = create_conn(baseurl, conn)
+	conn,protocol,address,params,headers = create_conn(baseurl, conn)
 
 	sys.stderr.write("Fetching '"+str(os.path.basename(address)+"'\n"))
 	if protocol in ["http","https"]:
-		data,rc,msg = make_http_request(conn, address, dest=dest)
+		data,rc,msg = make_http_request(conn, address, params, headers, dest=dest)
 	elif protocol in ["ftp"]:
 		data,rc,msg = make_ftp_request(conn, address, dest=dest)
 	else:
@@ -388,7 +399,7 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 	if makepickle == None:
 		makepickle = "/var/cache/edb/metadata.idx.most_recent"
 
-	conn,protocol,address = create_conn(baseurl, conn)
+	conn,protocol,address,params,headers = create_conn(baseurl, conn)
 
 	filedict = {}
 
@@ -469,12 +480,12 @@ def dir_get_metadata(baseurl,conn=None, chunk_size=3000, verbose=1, usingcache=1
 				sys.stderr.write("!!! "+str(e)+"\n")
 			break
 	# We may have metadata... now we run through the tbz2 list and check.
-	sys.stderr.write(red("cache miss: 'x'")+" --- "+green("cache hit: 'o'")+"\n")
+	sys.stderr.write(yellow("cache miss: 'x'")+" --- "+green("cache hit: 'o'")+"\n")
 	for x in tbz2list:
 		x = os.path.basename(x)
 		if ((not metadata[baseurl]["data"].has_key(x)) or \
 		    (x not in metadata[baseurl]["data"].keys())):
-			sys.stderr.write(red("x"))
+			sys.stderr.write(yellow("x"))
 			metadata[baseurl]["modified"] = 1
 			myid = file_get_metadata(baseurl+"/"+x, conn, chunk_size)
 		
