@@ -25,7 +25,7 @@
  *  as some of the InstallWatch code was used.
  *
  *
- *  $Header: /home/cvsroot/gentoo-src/portage/src/sandbox-1.1/libsandbox.c,v 1.11 2003/09/28 08:37:19 azarah Exp $
+ *  $Header: /home/cvsroot/gentoo-src/portage/src/sandbox-1.1/libsandbox.c,v 1.16 2004/03/22 01:40:58 carpaski Exp $
  *
  */
 
@@ -281,6 +281,8 @@ canonicalize(const char *path, char *resolved_path)
 	int old_errno = errno;
 	char *retval;
 
+	*resolved_path = '\0';
+
 	/* If path == NULL, return or we get a segfault */
 	if (NULL == path) {
 		errno = EINVAL;
@@ -472,8 +474,7 @@ mkdir(const char *pathname, mode_t mode)
 	canonicalize_int(pathname, canonic);
 
 	/* Check if the directory exist, return EEXIST rather than failing */
-	lstat(canonic, &st);
-	if (0 == errno) {
+	if (0 == lstat(canonic, &st)) {
 		errno = EEXIST;
 		return errno;
 	}
@@ -564,11 +565,11 @@ rename(const char *oldpath, const char *newpath)
 	canonicalize_int(oldpath, old_canonic);
 	canonicalize_int(newpath, new_canonic);
 
-	if FUNCTION_SANDBOX_SAFE
-		("rename", new_canonic) {
+	if (FUNCTION_SANDBOX_SAFE("rename", old_canonic) &&
+			FUNCTION_SANDBOX_SAFE("rename", new_canonic)) {
 		check_dlsym(rename);
 		result = true_rename(oldpath, newpath);
-		}
+	}
 
 	return result;
 }
@@ -734,9 +735,11 @@ execve(const char *filename, char *const argv[], char *const envp[])
 	int old_errno = errno;
 	int result = -1;
 	int count = 0;
+	int env_len = 0;
 	char canonic[SB_PATH_MAX];
-	char *old_envp = NULL;
-	char *new_envp = NULL;
+	char **my_env = NULL;
+	/* We limit the size LD_PRELOAD can be here, but it should be enough */
+	char tmp_str[4096];
 
 	canonicalize_int(filename, canonic);
 
@@ -745,36 +748,54 @@ execve(const char *filename, char *const argv[], char *const envp[])
 		while (envp[count] != NULL) {
 			if (strstr(envp[count], "LD_PRELOAD=") == envp[count]) {
 				if (NULL != strstr(envp[count], sandbox_lib)) {
+					my_env = (char **) envp;
 					break;
 				} else {
+					int i = 0;
 					const int max_envp_len =
 							strlen(envp[count]) + strlen(sandbox_lib) + 1;
 
-					/* Backup envp[count], and set it to our own one which
-					 * contains sandbox_lib */
-					old_envp = envp[count];
-					new_envp = strndupa(old_envp, max_envp_len - 1);
+					/* Fail safe ... */
+					if (max_envp_len > 4096) {
+						fprintf(stderr, "sandbox:  max_envp_len too big!\n");
+						errno = ENOMEM;
+						return result;
+					}
+
+					/* Calculate envp size */
+					my_env = (char **) envp;
+					do
+						env_len += 1;
+					while (*my_env++);
+
+					my_env = (char **) malloc((env_len + 2) * sizeof (char *));
+					if (NULL == my_env) {
+						errno = ENOMEM;
+						return result;
+					}
+					/* Copy envp to my_env */
+					do
+						my_env[i] = envp[i];
+					while (envp[i++]);
+
+					/* Set tmp_str to envp[count] */
+					strncpy(tmp_str, envp[count], max_envp_len - 1);
 
 					/* LD_PRELOAD already have variables other than sandbox_lib,
-					 * thus we have to add sandbox_lib via a white space. */
-					if (0 != strcmp(envp[count], "LD_PRELOAD=")) {
-						strncpy(new_envp + strlen(old_envp), ":",
-										max_envp_len - strlen(new_envp));
-						strncpy(new_envp + strlen(old_envp) + 1, sandbox_lib,
-										max_envp_len - strlen(new_envp));
+					 * thus we have to add sandbox_lib seperated via a whitespace. */
+					if (0 != strncmp(envp[count], "LD_PRELOAD=", max_envp_len - 1)) {
+						strncat(tmp_str, " ", max_envp_len - strlen(tmp_str));
+						strncat(tmp_str, sandbox_lib, max_envp_len - strlen(tmp_str));
 					} else {
-						strncpy(new_envp +
-										strlen(old_envp), sandbox_lib,
-										max_envp_len - strlen(new_envp));
+						strncat(tmp_str, sandbox_lib, max_envp_len - strlen(tmp_str));
 					}
 
 					/* Valid string? */
-					new_envp[max_envp_len] = '\0';
+					tmp_str[max_envp_len] = '\0';
 
-					/* envp[count] = new_envp;
-					 *
-					 * Get rid of the "read-only" warnings */
-					memcpy((void *) &envp[count], &new_envp, sizeof (new_envp));
+					/* Ok, replace my_env[count] with our version that contains
+					 * sandbox_lib ... */
+					my_env[count] = tmp_str;
 
 					break;
 				}
@@ -784,17 +805,14 @@ execve(const char *filename, char *const argv[], char *const envp[])
 
 		errno = old_errno;
 		check_dlsym(execve);
-		result = true_execve(filename, argv, envp);
+		result = true_execve(filename, argv, my_env);
 		old_errno = errno;
 
-		if (old_envp) {
-			/* Restore envp[count] again.
-			 * 
-			 * envp[count] = old_envp; */
-			memcpy((void *) &envp[count], &old_envp, sizeof (old_envp));
-			old_envp = NULL;
-		}
+		if (my_env) {
+			free(my_env);
+			my_env = NULL;
 		}
+	}
 
 	errno = old_errno;
 
@@ -1283,6 +1301,12 @@ before_syscall(const char *func, const char *file)
 	int result = 1;
 	sbcontext_t sbcontext;
 
+	if (!strlen(file)) {
+		/* The file/directory does not exist */
+		errno = ENOENT;
+		return 0;
+	}
+
 	init_context(&sbcontext);
 
 	init_env_entries(&(sbcontext.deny_prefixes),
